# -*- coding: utf-8 -*-

from django.conf.urls import patterns, url

urlpatterns = patterns(
    '',
    url(r'^$', 'watchman.views.status', name="status"),
    url(r'^dashboard/$', 'watchman.views.dashboard', name="dashboard"),
)



from django.http import HttpResponseForbidden
from django.views.decorators.csrf import csrf_exempt

from functools import wraps

from watchman import settings


def token_required(view_func):
    """
    Decorator which ensures that WATCHMAN_TOKEN is provided if set.

    WATCHMAN_TOKEN_NAME can also be set if the token GET parameter must be
    customized.

    """

    def _validate_token(request):
        watchman_token = settings.WATCHMAN_TOKEN
        if watchman_token is None:
            return True

        watchman_token_name = settings.WATCHMAN_TOKEN_NAME
        return watchman_token == request.GET.get(watchman_token_name)

    @csrf_exempt
    @wraps(view_func)
    def _wrapped_view(request, *args, **kwargs):
        if _validate_token(request):
            return view_func(request, *args, **kwargs)

        return HttpResponseForbidden()

    return _wrapped_view

if settings.WATCHMAN_AUTH_DECORATOR is None:
    def auth(view_func):
        @csrf_exempt
        @wraps(view_func)
        def _wrapped_view(request, *args, **kwargs):
            return view_func(request, *args, **kwargs)

        return _wrapped_view
elif settings.WATCHMAN_AUTH_DECORATOR == 'watchman.decorators.token_required':
    # Avoid import loops
    auth = token_required
else:
    try:
        from importlib import import_module
    except ImportError:  # Django < 1.8
        from django.utils.importlib import import_module

    mod_name, dec = settings.WATCHMAN_AUTH_DECORATOR.rsplit('.', 1)
    auth = getattr(import_module(mod_name), dec)



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

import traceback
import uuid
from django.conf import settings
from django.core.cache import get_cache
from django.core.files.base import ContentFile
from django.core.files.storage import default_storage
from django.core.mail import EmailMessage
from django.db import connections


def _check_caches(caches):
    return [_check_cache(cache) for cache in caches]


def _check_cache(cache_name):
    key = 'django-watchman-{}'.format(uuid.uuid4())
    value = 'django-watchman-{}'.format(uuid.uuid4())
    try:
        cache = get_cache(cache_name)
        cache.set(key, value)
        cache.get(key)
        cache.delete(key)
        response = {cache_name: {"ok": True}}
    except Exception as e:
        response = {
            cache_name: {
                "ok": False,
                "error": str(e),
                "stacktrace": traceback.format_exc(),
            },
        }
    return response


def _check_databases(databases):
    return [_check_database(database) for database in databases]


def _check_database(database):
    try:
        connections[database].introspection.table_names()
        response = {database: {"ok": True}}
    except Exception as e:
        response = {
            database: {
                "ok": False,
                "error": str(e),
                "stacktrace": traceback.format_exc(),
            },
        }
    return response


def _check_email():
    try:
        headers = {"X-DJANGO-WATCHMAN": True}
        email = EmailMessage(
            "django-watchman email check",
            "This is an automated test of the email system.",
            "watchman@example.com",
            ["to@example.com"],
            headers=headers
        )
        email.send()
        response = {"ok": True}
    except Exception as e:
        response = {
            "ok": False,
            "error": str(e),
            "stacktrace": traceback.format_exc(),
        }
    return response


def _check_storage():
    try:
        filename = 'django-watchman-{}.txt'.format(uuid.uuid4())
        content = 'django-watchman test file'
        path = default_storage.save(filename, ContentFile(content))
        default_storage.size(path)
        default_storage.open(path).read()
        default_storage.delete(path)
        response = {"ok": True}
    except Exception as e:
        response = {
            "ok": False,
            "error": str(e),
            "stacktrace": traceback.format_exc(),
        }
    return response


def caches():
    return {"caches": _check_caches(settings.CACHES)}


def databases():
    return {"databases": _check_databases(settings.DATABASES)}


def email():
    return {"email": _check_email()}


def storage():
    return {"storage": _check_storage()}



from django.conf import settings

# TODO: these should not be module level.
WATCHMAN_ENABLE_PAID_CHECKS = getattr(settings, 'WATCHMAN_ENABLE_PAID_CHECKS', False)
WATCHMAN_AUTH_DECORATOR = getattr(settings, 'WATCHMAN_AUTH_DECORATOR', 'watchman.decorators.token_required')
WATCHMAN_TOKEN = getattr(settings, 'WATCHMAN_TOKEN', None)
WATCHMAN_TOKEN_NAME = getattr(settings, 'WATCHMAN_TOKEN_NAME', 'watchman-token')

DEFAULT_CHECKS = (
    'watchman.checks.caches',
    'watchman.checks.databases',
    'watchman.checks.storage',
)
PAID_CHECKS = (
    'watchman.checks.email',
)

if WATCHMAN_ENABLE_PAID_CHECKS:
    DEFAULT_CHECKS = DEFAULT_CHECKS + PAID_CHECKS

WATCHMAN_CHECKS = getattr(settings, 'WATCHMAN_CHECKS', DEFAULT_CHECKS)
